@@ -86,7 +86,7 @@ def register(
     """Registers input channel blueprints with Sanic."""
 
     async def handler(message: UserMessage) -> None:
-        await app.ctx.agent.handle_message(message)
+        await app.agent.handle_message(message)
 
     for channel in input_channels:
         if route:
@@ -95,7 +95,7 @@ def register(
             p = None
         app.blueprint(channel.blueprint(handler), url_prefix=p)
 
-    app.ctx.input_channels = input_channels
+    app.input_channels = input_channels
 
 
 class InputChannel:
@@ -200,6 +200,8 @@ def decode_bearer_token(
     except Exception:
         logger.exception("Failed to decode bearer token.")
 
+    return None
+
 
 class OutputChannel:
     """Output channel base class.
@@ -393,21 +395,11 @@ class CollectingOutputChannel(OutputChannel):
         buttons: List[Dict[Text, Any]],
         **kwargs: Any,
     ) -> None:
-        """Sends text message with buttons."""
-        message_parts: List[Text] = text.strip().split("\n\n")
-        last_message_idx: int = len(message_parts) - 1
-
-        for idx, message_part in enumerate(message_parts):
-            await self._persist_message(
-                self._message(
-                    recipient_id,
-                    text=message_part,
-                    buttons=buttons if idx == last_message_idx else None,
-                )
-            )
+        await self._persist_message(
+            self._message(recipient_id, text=text, buttons=buttons)
+        )
 
     async def send_custom_json(
         self, recipient_id: Text, json_message: Dict[Text, Any], **kwargs: Any
     ) -> None:
-        """Sends custom json."""
         await self._persist_message(self._message(recipient_id, custom=json_message))
