@@ -10,7 +10,7 @@ from typing import Text, List, Dict, Any, Optional, Callable, Iterable, Awaitabl
 
 from google.oauth2 import id_token
 from sanic.response import HTTPResponse
-from sanic.exceptions import SanicException
+from sanic.exceptions import abort
 
 from rasa.core.channels.channel import InputChannel, OutputChannel, UserMessage
 
@@ -66,7 +66,7 @@ class HangoutsOutput(OutputChannel):
                 logger.error(
                     "Buttons must be a list of dicts with 'title' and 'payload' as keys"
                 )
-                return
+                return None
 
             hangouts_buttons.append(
                 {
@@ -260,10 +260,11 @@ class HangoutsInput(InputChannel):
 
     @staticmethod
     def _extract_room(req: Request) -> Union[Text, None]:
-
         if req.json["space"]["type"] == "ROOM":
             return req.json["space"]["displayName"]
 
+        return None
+
     def _extract_input_channel(self) -> Text:
         return self.name()
 
@@ -278,9 +279,9 @@ class HangoutsInput(InputChannel):
                 certs_url=CERTS_URL,
             )
         except ValueError:
-            raise SanicException(status_code=401)
+            abort(401)
         if decoded_token["iss"] != "chat@system.gserviceaccount.com":
-            raise SanicException(status_code=401)
+            abort(401)
 
     def blueprint(
         self, on_new_message: Callable[[UserMessage], Awaitable[None]]
