@@ -1,81 +1,95 @@
+from __future__ import annotations
 import logging
 import re
-from typing import Any, Dict, List, Optional, Text, Type, Tuple
-from pathlib import Path
+from typing import Any, Dict, List, Optional, Text, Tuple, Type
 import numpy as np
 import scipy.sparse
+from rasa.nlu.tokenizers.tokenizer import Tokenizer
 
 import rasa.shared.utils.io
 import rasa.utils.io
 import rasa.nlu.utils.pattern_utils as pattern_utils
-from rasa.nlu import utils
-from rasa.nlu.components import Component
-from rasa.nlu.config import RasaNLUModelConfig
-from rasa.nlu.constants import (
-    TOKENS_NAMES,
-    FEATURIZER_CLASS_ALIAS,
-)
+from rasa.engine.graph import ExecutionContext, GraphComponent
+from rasa.engine.recipes.default_recipe import DefaultV1Recipe
+from rasa.engine.storage.resource import Resource
+from rasa.engine.storage.storage import ModelStorage
+from rasa.nlu.constants import TOKENS_NAMES
+from rasa.nlu.featurizers.sparse_featurizer.sparse_featurizer import SparseFeaturizer
 from rasa.shared.nlu.constants import (
     TEXT,
     RESPONSE,
-    FEATURE_TYPE_SENTENCE,
-    FEATURE_TYPE_SEQUENCE,
     ACTION_TEXT,
 )
-from rasa.nlu.featurizers.featurizer import SparseFeaturizer
-from rasa.shared.nlu.training_data.features import Features
-from rasa.nlu.model import Metadata
-from rasa.nlu.tokenizers.tokenizer import Tokenizer
 from rasa.shared.nlu.training_data.training_data import TrainingData
 from rasa.shared.nlu.training_data.message import Message
 
 logger = logging.getLogger(__name__)
 
 
-class RegexFeaturizer(SparseFeaturizer):
+@DefaultV1Recipe.register(
+    DefaultV1Recipe.ComponentType.MESSAGE_FEATURIZER, is_trainable=True
+)
+class RegexFeaturizer(SparseFeaturizer, GraphComponent):
+    """Adds message features based on regex expressions."""
+
     @classmethod
-    def required_components(cls) -> List[Type[Component]]:
+    def required_components(cls) -> List[Type]:
+        """Components that should be included in the pipeline before this component."""
         return [Tokenizer]
 
-    defaults = {
-        # text will be processed with case sensitive as default
-        "case_sensitive": True,
-        # use lookup tables to generate features
-        "use_lookup_tables": True,
-        # use regexes to generate features
-        "use_regexes": True,
-        # use match word boundaries for lookup table
-        "use_word_boundaries": True,
-        # Additional number of patterns to consider
-        # for incremental training
-        "number_additional_patterns": None,
-    }
+    @staticmethod
+    def get_default_config() -> Dict[Text, Any]:
+        """Returns the component's default config."""
+        return {
+            **SparseFeaturizer.get_default_config(),
+            # text will be processed with case sensitive as default
+            "case_sensitive": True,
+            # use lookup tables to generate features
+            "use_lookup_tables": True,
+            # use regexes to generate features
+            "use_regexes": True,
+            # use match word boundaries for lookup table
+            "use_word_boundaries": True,
+        }
 
     def __init__(
         self,
-        component_config: Optional[Dict[Text, Any]] = None,
+        config: Dict[Text, Any],
+        model_storage: ModelStorage,
+        resource: Resource,
+        execution_context: ExecutionContext,
         known_patterns: Optional[List[Dict[Text, Text]]] = None,
-        finetune_mode: bool = False,
     ) -> None:
         """Constructs new features for regexes and lookup table using regex expressions.
 
         Args:
-            component_config: Configuration for the component
+            config: Configuration for the component.
+            model_storage: Storage which graph components can use to persist and load
+                themselves.
+            resource: Resource locator for this component which can be used to persist
+                and load itself from the `model_storage`.
+            execution_context: Information about the current graph run.
             known_patterns: Regex Patterns the component should pre-load itself with.
-            finetune_mode: Load component in finetune mode.
         """
-        super().__init__(component_config)
+        super().__init__(execution_context.node_name, config)
+
+        self._model_storage = model_storage
+        self._resource = resource
 
         self.known_patterns = known_patterns if known_patterns else []
-        self.case_sensitive = self.component_config["case_sensitive"]
-        self.finetune_mode = finetune_mode
-        if self.component_config["number_additional_patterns"]:
-            rasa.shared.utils.io.raise_deprecation_warning(
-                "The parameter `number_additional_patterns` has been deprecated "
-                "since the pipeline does not create an extra buffer for new vocabulary "
-                "anymore. Any value assigned to this parameter will be ignored. "
-                "You can omit specifying `number_additional_patterns` in future runs."
-            )
+        self.case_sensitive = config["case_sensitive"]
+        self.finetune_mode = execution_context.is_finetuning
+
+    @classmethod
+    def create(
+        cls,
+        config: Dict[Text, Any],
+        model_storage: ModelStorage,
+        resource: Resource,
+        execution_context: ExecutionContext,
+    ) -> RegexFeaturizer:
+        """Creates a new untrained component (see parent class for full docstring)."""
+        return cls(config, model_storage, resource, execution_context)
 
     def _merge_new_patterns(self, new_patterns: List[Dict[Text, Text]]) -> None:
         """Updates already known patterns with new patterns extracted from data.
@@ -102,25 +116,13 @@ class RegexFeaturizer(SparseFeaturizer):
             else:
                 self.known_patterns.append(extra_pattern)
 
-    def train(
-        self,
-        training_data: TrainingData,
-        config: Optional[RasaNLUModelConfig] = None,
-        **kwargs: Any,
-    ) -> None:
-        """Trains the component with all patterns extracted from training data.
-
-        Args:
-            training_data: Training data consisting of training examples and patterns
-                available.
-            config: NLU Pipeline config
-            **kwargs: Any other arguments
-        """
+    def train(self, training_data: TrainingData,) -> Resource:
+        """Trains the component with all patterns extracted from training data."""
         patterns_from_data = pattern_utils.extract_patterns(
             training_data,
-            use_lookup_tables=self.component_config["use_lookup_tables"],
-            use_regexes=self.component_config["use_regexes"],
-            use_word_boundaries=self.component_config["use_word_boundaries"],
+            use_lookup_tables=self._config["use_lookup_tables"],
+            use_regexes=self._config["use_regexes"],
+            use_word_boundaries=self._config["use_word_boundaries"],
         )
         if self.finetune_mode:
             # Merge patterns extracted from data with known patterns
@@ -128,12 +130,27 @@ class RegexFeaturizer(SparseFeaturizer):
         else:
             self.known_patterns = patterns_from_data
 
+        self._persist()
+        return self._resource
+
+    def process_training_data(self, training_data: TrainingData) -> TrainingData:
+        """Processes the training examples (see parent class for full docstring)."""
         for example in training_data.training_examples:
             for attribute in [TEXT, RESPONSE, ACTION_TEXT]:
                 self._text_features_with_regex(example, attribute)
 
-    def process(self, message: Message, **kwargs: Any) -> None:
-        self._text_features_with_regex(message, TEXT)
+        return training_data
+
+    def process(self, messages: List[Message]) -> List[Message]:
+        """Featurizes all given messages in-place.
+
+        Returns:
+          the given list of messages which have been modified in-place
+        """
+        for message in messages:
+            self._text_features_with_regex(message, TEXT)
+
+        return messages
 
     def _text_features_with_regex(self, message: Message, attribute: Text) -> None:
         """Helper method to extract features and set them appropriately in the message.
@@ -147,23 +164,9 @@ class RegexFeaturizer(SparseFeaturizer):
                 message, attribute
             )
 
-            if sequence_features is not None:
-                final_sequence_features = Features(
-                    sequence_features,
-                    FEATURE_TYPE_SEQUENCE,
-                    attribute,
-                    self.component_config[FEATURIZER_CLASS_ALIAS],
-                )
-                message.add_features(final_sequence_features)
-
-            if sentence_features is not None:
-                final_sentence_features = Features(
-                    sentence_features,
-                    FEATURE_TYPE_SENTENCE,
-                    attribute,
-                    self.component_config[FEATURIZER_CLASS_ALIAS],
-                )
-                message.add_features(final_sentence_features)
+            self.add_features_to_message(
+                sequence_features, sentence_features, attribute, message
+            )
 
     def _features_for_patterns(
         self, message: Message, attribute: Text
@@ -222,6 +225,7 @@ class RegexFeaturizer(SparseFeaturizer):
                             sentence_features[0][pattern_index] = 1.0
 
                 t.set("pattern", patterns)
+
         return (
             scipy.sparse.coo_matrix(sequence_features),
             scipy.sparse.coo_matrix(sentence_features),
@@ -230,48 +234,41 @@ class RegexFeaturizer(SparseFeaturizer):
     @classmethod
     def load(
         cls,
-        meta: Dict[Text, Any],
-        model_dir: Text,
-        model_metadata: Optional[Metadata] = None,
-        cached_component: Optional["RegexFeaturizer"] = None,
-        should_finetune: bool = False,
+        config: Dict[Text, Any],
+        model_storage: ModelStorage,
+        resource: Resource,
+        execution_context: ExecutionContext,
         **kwargs: Any,
-    ) -> "RegexFeaturizer":
-        """Loads a previously trained component.
-
-        Args:
-            meta: Configuration of trained component.
-            model_dir: Path where trained pipeline is stored.
-            model_metadata: Metadata for the trained pipeline.
-            cached_component: Previously cached component(if any).
-            should_finetune: Indicates whether to load the component for further
-                finetuning.
-            **kwargs: Any other arguments.
-        """
-        file_name = meta.get("file")
-
-        patterns_file_name = Path(model_dir) / (file_name + ".patterns.pkl")
-
+    ) -> RegexFeaturizer:
+        """Loads trained component (see parent class for full docstring)."""
         known_patterns = None
-        if patterns_file_name.exists():
-            known_patterns = rasa.shared.utils.io.read_json_file(patterns_file_name)
 
-        return RegexFeaturizer(
-            meta, known_patterns=known_patterns, finetune_mode=should_finetune,
+        try:
+            with model_storage.read_from(resource) as model_dir:
+                patterns_file_name = model_dir / "patterns.pkl"
+                known_patterns = rasa.shared.utils.io.read_json_file(patterns_file_name)
+        except (ValueError, FileNotFoundError):
+            logger.warning(
+                f"Failed to load `{cls.__class__.__name__}` from model storage. "
+                f"Resource '{resource.name}' doesn't exist."
+            )
+
+        return cls(
+            config,
+            model_storage,
+            resource,
+            execution_context,
+            known_patterns=known_patterns,
         )
 
-    def persist(self, file_name: Text, model_dir: Text) -> Optional[Dict[Text, Any]]:
-        """Persist this model into the passed directory.
+    def _persist(self) -> None:
+        with self._model_storage.write_to(self._resource) as model_dir:
+            regex_file = model_dir / "patterns.pkl"
+            rasa.shared.utils.io.dump_obj_as_json_to_file(
+                regex_file, self.known_patterns
+            )
 
-        Args:
-            file_name: Prefix to add to all files stored as part of this component.
-            model_dir: Path where files should be stored.
-
-        Returns:
-            Metadata necessary to load the model again.
-        """
-        patterns_file_name = file_name + ".patterns.pkl"
-        regex_file = Path(model_dir) / patterns_file_name
-        utils.write_json_to_file(regex_file, self.known_patterns, indent=4)
-
-        return {"file": file_name}
+    @classmethod
+    def validate_config(cls, config: Dict[Text, Any]) -> None:
+        """Validates that the component is configured properly."""
+        pass
