@@ -33,6 +33,7 @@ from sanic_jwt import Initialize, exceptions
 
 import rasa
 import rasa.core.utils
+from rasa.nlu.emulators.emulator import Emulator
 import rasa.utils.common
 import rasa.shared.utils.common
 import rasa.shared.utils.io
@@ -43,20 +44,16 @@ from rasa.shared.core.training_data.story_writer.yaml_story_writer import (
 )
 from rasa.shared.importers.importer import TrainingDataImporter
 from rasa.shared.nlu.training_data.formats import RasaYAMLReader
-from rasa import model
 from rasa.constants import DEFAULT_RESPONSE_TIMEOUT, MINIMUM_COMPATIBLE_VERSION
 from rasa.shared.constants import (
     DOCS_URL_TRAINING_DATA,
     DOCS_BASE_URL,
     DEFAULT_SENDER_ID,
-    DEFAULT_DOMAIN_PATH,
     DEFAULT_MODELS_PATH,
-    DEFAULT_CONVERSATION_TEST_PATH,
     TEST_STORIES_FILE_PREFIX,
 )
 from rasa.shared.core.domain import InvalidDomain, Domain
 from rasa.core.agent import Agent
-from rasa.core.brokers.broker import EventBroker
 from rasa.core.channels.channel import (
     CollectingOutputChannel,
     OutputChannel,
@@ -64,9 +61,7 @@ from rasa.core.channels.channel import (
 )
 import rasa.shared.core.events
 from rasa.shared.core.events import Event
-from rasa.core.lock_store import LockStore
 from rasa.core.test import test
-from rasa.core.tracker_store import TrackerStore
 from rasa.shared.core.trackers import DialogueStateTracker, EventVerbosity
 from rasa.core.utils import AvailableEndpoints
 from rasa.nlu.emulators.no_emulator import NoEmulator
@@ -78,9 +73,14 @@ from rasa.utils.endpoints import EndpointConfig
 if TYPE_CHECKING:
     from ssl import SSLContext  # noqa: F401
     from rasa.core.processor import MessageProcessor
-    from mypy_extensions import VarArg, KwArg
+    from mypy_extensions import Arg, VarArg, KwArg
 
-    SanicView = Callable[[Request, VarArg(), KwArg()], response.BaseHTTPResponse]
+    SanicResponse = Union[
+        response.HTTPResponse, Coroutine[Any, Any, response.HTTPResponse]
+    ]
+    SanicView = Callable[
+        [Arg(Request, "request"), VarArg(), KwArg()], SanicResponse,  # noqa: F821
+    ]
 
 
 logger = logging.getLogger(__name__)
@@ -145,11 +145,7 @@ def ensure_loaded_agent(
         @wraps(f)
         def decorated(*args: Any, **kwargs: Any) -> Any:
             # noinspection PyUnresolvedReferences
-            if not app.ctx.agent or not (
-                app.ctx.agent.is_core_ready()
-                if require_core_is_ready
-                else app.ctx.agent.is_ready()
-            ):
+            if not app.agent or not app.agent.is_ready():
                 raise ErrorResponse(
                     HTTPStatus.CONFLICT,
                     "Conflict",
@@ -165,14 +161,14 @@ def ensure_loaded_agent(
     return decorator
 
 
-def ensure_conversation_exists() -> "SanicView":
+def ensure_conversation_exists() -> Callable[["SanicView"], "SanicView"]:
     """Wraps a request handler ensuring the conversation exists."""
 
-    def decorator(f: "SanicView") -> HTTPResponse:
+    def decorator(f: "SanicView") -> "SanicView":
         @wraps(f)
-        def decorated(request: Request, *args: Any, **kwargs: Any) -> HTTPResponse:
+        def decorated(request: Request, *args: Any, **kwargs: Any) -> "SanicResponse":
             conversation_id = kwargs["conversation_id"]
-            if request.app.ctx.agent.tracker_store.exists(conversation_id):
+            if request.app.agent.tracker_store.exists(conversation_id):
                 return f(request, *args, **kwargs)
             else:
                 raise ErrorResponse(
@@ -227,16 +223,14 @@ def requires_auth(
         @wraps(f)
         async def decorated(
             request: Request, *args: Any, **kwargs: Any
-        ) -> response.BaseHTTPResponse:
+        ) -> response.HTTPResponse:
 
             provided = request.args.get("token", None)
 
             # noinspection PyProtectedMember
             if token is not None and provided == token:
                 result = f(request, *args, **kwargs)
-                if isawaitable(result):
-                    result = await result
-                return result
+                return await result if isawaitable(result) else result
             elif app.config.get(
                 "USE_JWT"
             ) and await rasa.utils.common.call_potential_coroutine(
@@ -245,9 +239,7 @@ def requires_auth(
             ):
                 if await sufficient_scope(request, *args, **kwargs):
                     result = f(request, *args, **kwargs)
-                    if isawaitable(result):
-                        result = await result
-                    return result
+                    return await result if isawaitable(result) else result
                 raise ErrorResponse(
                     HTTPStatus.FORBIDDEN,
                     "NotAuthorized",
@@ -259,9 +251,7 @@ def requires_auth(
             elif token is None and app.config.get("USE_JWT") is None:
                 # authentication is disabled
                 result = f(request, *args, **kwargs)
-                if isawaitable(result):
-                    result = await result
-                return result
+                return await result if isawaitable(result) else result
             raise ErrorResponse(
                 HTTPStatus.UNAUTHORIZED,
                 "NotAuthenticated",
@@ -440,10 +430,11 @@ def create_ssl_context(
         return None
 
 
-def _create_emulator(mode: Optional[Text]) -> NoEmulator:
+def _create_emulator(mode: Optional[Text]) -> Emulator:
     """Create emulator for specified mode.
-    If no emulator is specified, we will use the Rasa NLU format."""
 
+    If no emulator is specified, we will use the Rasa NLU format.
+    """
     if mode is None:
         return NoEmulator()
     elif mode.lower() == "wit":
@@ -473,31 +464,13 @@ async def _load_agent(
     model_server: Optional[EndpointConfig] = None,
     remote_storage: Optional[Text] = None,
     endpoints: Optional[AvailableEndpoints] = None,
-    lock_store: Optional[LockStore] = None,
 ) -> Agent:
     try:
-        tracker_store = None
-        generator = None
-        action_endpoint = None
-
-        if endpoints:
-            broker = await EventBroker.create(endpoints.event_broker)
-            tracker_store = TrackerStore.create(
-                endpoints.tracker_store, event_broker=broker
-            )
-            generator = endpoints.nlg
-            action_endpoint = endpoints.action
-            if not lock_store:
-                lock_store = LockStore.create(endpoints.lock_store)
-
         loaded_agent = await rasa.core.agent.load_agent(
-            model_path,
-            model_server,
-            remote_storage,
-            generator=generator,
-            tracker_store=tracker_store,
-            lock_store=lock_store,
-            action_endpoint=action_endpoint,
+            model_path=model_path,
+            model_server=model_server,
+            remote_storage=remote_storage,
+            endpoints=endpoints,
         )
     except Exception as e:
         logger.debug(traceback.format_exc())
@@ -507,7 +480,7 @@ async def _load_agent(
             f"An unexpected error occurred. Error: {e}",
         )
 
-    if not loaded_agent:
+    if not loaded_agent.is_ready():
         raise ErrorResponse(
             HTTPStatus.BAD_REQUEST,
             "BadRequest",
@@ -571,7 +544,7 @@ def async_if_callback_url(f: Callable[..., Coroutine]) -> Callable:
         async def wrapped() -> None:
             try:
                 result: HTTPResponse = await f(request, *args, **kwargs)
-                payload = dict(
+                payload: Dict[Text, Any] = dict(
                     data=result.body, headers={"Content-Type": result.content_type}
                 )
                 logger.debug(
@@ -588,7 +561,7 @@ def async_if_callback_url(f: Callable[..., Coroutine]) -> Callable:
                         f"An unexpected error occurred. Error: {e}",
                     )
                 # If an error happens, we send the error payload to the `callback_url`
-                payload = dict(json=e.error_info)  # type: ignore[dict-item]
+                payload = dict(json=e.error_info)
                 logger.error(
                     "Error happened when processing request asynchronously. "
                     "Sending error to callback URL."
@@ -627,14 +600,7 @@ def run_in_thread(f: Callable[..., Coroutine]) -> Callable:
         # Use a sync wrapper for our `async` function as `run_in_executor` only supports
         # sync functions
         def run() -> HTTPResponse:
-            # This is a new thread, so we need to create and set a new event loop
-            thread_loop = asyncio.new_event_loop()
-            asyncio.set_event_loop(thread_loop)
-
-            try:
-                return thread_loop.run_until_complete(f(request, *args, **kwargs))
-            finally:
-                thread_loop.close()
+            return asyncio.run(f(request, *args, **kwargs))
 
         with concurrent.futures.ThreadPoolExecutor() as pool:
             return await request.app.loop.run_in_executor(pool, run)
@@ -671,7 +637,7 @@ def create_app(
     endpoints: Optional[AvailableEndpoints] = None,
 ) -> Sanic:
     """Class representing a Rasa HTTP server."""
-    app = Sanic("rasa_server", register=False)
+    app = Sanic(__name__)
     app.config.RESPONSE_TIMEOUT = response_timeout
     configure_cors(app, cors_origins)
 
@@ -689,10 +655,10 @@ def create_app(
             user_id="username",
         )
 
-    app.ctx.agent = agent
+    app.agent = agent
     # Initialize shared object of type unsigned int for tracking
     # the number of active training processes
-    app.ctx.active_training_processes = multiprocessing.Value("I", 0)
+    app.active_training_processes = multiprocessing.Value("I", 0)
 
     @app.exception(ErrorResponse)
     async def handle_error_response(
@@ -718,15 +684,11 @@ def create_app(
     @ensure_loaded_agent(app)
     async def status(request: Request) -> HTTPResponse:
         """Respond with the model name and the fingerprint of that model."""
-
         return response.json(
             {
-                "model_file": app.ctx.agent.path_to_model_archive
-                or app.ctx.agent.model_directory,
-                "fingerprint": model.fingerprint_from_path(
-                    app.ctx.agent.model_directory
-                ),
-                "num_active_training_jobs": app.ctx.active_training_processes.value,
+                "model_file": app.agent.processor.model_filename,
+                "model_id": app.agent.model_id,
+                "num_active_training_jobs": app.active_training_processes.value,
             }
         )
 
@@ -738,7 +700,7 @@ def create_app(
         verbosity = event_verbosity_parameter(request, EventVerbosity.AFTER_RESTART)
         until_time = rasa.utils.endpoints.float_arg(request, "until")
 
-        tracker = await app.ctx.agent.create_processor().fetch_tracker_with_initial_session(  # noqa: E501
+        tracker = await app.agent.processor.fetch_tracker_with_initial_session(
             conversation_id
         )
 
@@ -766,12 +728,12 @@ def create_app(
         verbosity = event_verbosity_parameter(request, EventVerbosity.AFTER_RESTART)
 
         try:
-            async with app.ctx.agent.lock_store.lock(conversation_id):
-                processor = app.ctx.agent.create_processor()
+            async with app.agent.lock_store.lock(conversation_id):
+                processor = app.agent.processor
                 events = _get_events_from_request_body(request)
 
                 tracker = await update_conversation_with_events(
-                    conversation_id, processor, app.ctx.agent.domain, events
+                    conversation_id, processor, app.agent.domain, events
                 )
 
                 output_channel = _get_output_channel(request, tracker)
@@ -783,7 +745,7 @@ def create_app(
                         events, tracker, output_channel
                     )
 
-                app.ctx.agent.tracker_store.save(tracker)
+                app.agent.tracker_store.save(tracker)
 
             return response.json(tracker.current_state(verbosity))
         except Exception as e:
@@ -827,13 +789,13 @@ def create_app(
         verbosity = event_verbosity_parameter(request, EventVerbosity.AFTER_RESTART)
 
         try:
-            async with app.ctx.agent.lock_store.lock(conversation_id):
+            async with app.agent.lock_store.lock(conversation_id):
                 tracker = DialogueStateTracker.from_dict(
-                    conversation_id, request.json, app.ctx.agent.domain.slots
+                    conversation_id, request.json, app.agent.domain.slots
                 )
 
                 # will override an existing tracker with the same id!
-                app.ctx.agent.tracker_store.save(tracker)
+                app.agent.tracker_store.save(tracker)
 
             return response.json(tracker.current_state(verbosity))
         except Exception as e:
@@ -857,7 +819,7 @@ def create_app(
 
         try:
             stories = get_test_stories(
-                app.ctx.agent.create_processor(),
+                app.agent.processor,
                 conversation_id,
                 until_time,
                 fetch_all_sessions=fetch_all_sessions,
@@ -893,15 +855,15 @@ def create_app(
         verbosity = event_verbosity_parameter(request, EventVerbosity.AFTER_RESTART)
 
         try:
-            async with app.ctx.agent.lock_store.lock(conversation_id):
+            async with app.agent.lock_store.lock(conversation_id):
                 tracker = await (
-                    app.ctx.agent.create_processor().fetch_tracker_and_update_session(
+                    app.agent.processor.fetch_tracker_and_update_session(
                         conversation_id
                     )
                 )
 
                 output_channel = _get_output_channel(request, tracker)
-                await app.ctx.agent.execute_action(
+                await app.agent.execute_action(
                     conversation_id,
                     action_to_execute,
                     output_channel,
@@ -946,20 +908,20 @@ def create_app(
         verbosity = event_verbosity_parameter(request, EventVerbosity.AFTER_RESTART)
 
         try:
-            async with app.ctx.agent.lock_store.lock(conversation_id):
+            async with app.agent.lock_store.lock(conversation_id):
                 tracker = await (
-                    app.ctx.agent.create_processor().fetch_tracker_and_update_session(
+                    app.agent.processor.fetch_tracker_and_update_session(
                         conversation_id
                     )
                 )
                 output_channel = _get_output_channel(request, tracker)
-                if intent_to_trigger not in app.ctx.agent.domain.intents:
+                if intent_to_trigger not in app.agent.domain.intents:
                     raise ErrorResponse(
                         HTTPStatus.NOT_FOUND,
                         "NotFound",
                         f"The intent {trigger_intent} does not exist in the domain.",
                     )
-                await app.ctx.agent.trigger_intent(
+                await app.agent.trigger_intent(
                     intent_name=intent_to_trigger,
                     entities=entities,
                     output_channel=output_channel,
@@ -991,7 +953,7 @@ def create_app(
     async def predict(request: Request, conversation_id: Text) -> HTTPResponse:
         try:
             # Fetches the appropriate bot response in a json format
-            responses = await app.ctx.agent.predict_next(conversation_id)
+            responses = await app.agent.predict_next_for_sender_id(conversation_id)
             responses["scores"] = sorted(
                 responses["scores"], key=lambda k: (-k["score"], k["action"])
             )
@@ -1035,8 +997,15 @@ def create_app(
         user_message = UserMessage(message, None, conversation_id, parse_data)
 
         try:
-            async with app.ctx.agent.lock_store.lock(conversation_id):
-                tracker = await app.ctx.agent.log_message(user_message)
+            async with app.agent.lock_store.lock(conversation_id):
+                # cf. processor.handle_message (ignoring prediction loop run)
+                tracker = await app.agent.processor.log_message(
+                    user_message, should_save_tracker=False
+                )
+                tracker = await app.agent.processor.run_action_extract_slots(
+                    user_message.output_channel, tracker
+                )
+                app.agent.processor.save_tracker(tracker)
 
             return response.json(tracker.current_state(verbosity))
         except Exception as e:
@@ -1059,19 +1028,16 @@ def create_app(
             "train your model.",
         )
 
-        if request.headers.get("Content-type") == YAML_CONTENT_TYPE:
-            training_payload = _training_payload_from_yaml(request, temporary_directory)
-        else:
-            training_payload = _training_payload_from_json(request, temporary_directory)
+        training_payload = _training_payload_from_yaml(request, temporary_directory)
 
         try:
-            with app.ctx.active_training_processes.get_lock():
-                app.ctx.active_training_processes.value += 1
+            with app.active_training_processes.get_lock():
+                app.active_training_processes.value += 1
 
-            from rasa.model_training import train_async
+            from rasa.model_training import train
 
             # pass `None` to run in default executor
-            training_result = await train_async(**training_payload)
+            training_result = train(**training_payload)
 
             if training_result.model:
                 filename = os.path.basename(training_result.model)
@@ -1103,8 +1069,8 @@ def create_app(
                 f"An unexpected error occurred during training. Error: {e}",
             )
         finally:
-            with app.ctx.active_training_processes.get_lock():
-                app.ctx.active_training_processes.value -= 1
+            with app.active_training_processes.get_lock():
+                app.active_training_processes.value -= 1
 
     @app.post("/model/test/stories")
     @requires_auth(app, auth_token)
@@ -1120,13 +1086,13 @@ def create_app(
             "evaluate your model.",
         )
 
-        test_data = _test_data_file_from_payload(request, temporary_directory, ".md")
+        test_data = _test_data_file_from_payload(request, temporary_directory)
 
-        use_e2e = rasa.utils.endpoints.bool_arg(request, "e2e", default=False)
+        e2e = rasa.utils.endpoints.bool_arg(request, "e2e", default=False)
 
         try:
             evaluation = await test(
-                test_data, app.ctx.agent, e2e=use_e2e, disable_plotting=True
+                test_data, app.agent, e2e=e2e, disable_plotting=True
             )
             return response.json(evaluation)
         except Exception as e:
@@ -1195,10 +1161,10 @@ def create_app(
     ) -> Dict:
         logger.info("Starting model evaluation using test set.")
 
-        eval_agent = app.ctx.agent
+        eval_agent = app.agent
 
         if model_path:
-            model_server = app.ctx.agent.model_server
+            model_server = app.agent.model_server
             if model_server is not None:
                 model_server = model_server.copy()
                 model_server.url = model_path
@@ -1206,28 +1172,20 @@ def create_app(
                 # a job to pull the model from the server
                 model_server.kwargs["wait_time_between_pulls"] = 0
             eval_agent = await _load_agent(
-                model_path, model_server, app.ctx.agent.remote_storage
+                model_path=model_path,
+                model_server=model_server,
+                remote_storage=app.agent.remote_storage,
             )
 
         data_path = os.path.abspath(test_data_file)
 
-        if not eval_agent.model_directory or not os.path.exists(
-            eval_agent.model_directory
-        ):
+        if not eval_agent.is_ready():
             raise ErrorResponse(
                 HTTPStatus.CONFLICT, "Conflict", "Loaded model file not found."
             )
 
-        model_directory = eval_agent.model_directory
-        _, nlu_model = model.get_model_subdirectories(model_directory)
-
-        if nlu_model is None:
-            raise ErrorResponse(
-                HTTPStatus.CONFLICT, "Conflict", "Missing NLU model directory.",
-            )
-
         return await rasa.nlu.test.run_evaluation(
-            data_path, nlu_model, disable_plotting=True, report_as_dict=True
+            data_path, eval_agent.processor, disable_plotting=True, report_as_dict=True
         )
 
     async def _cross_validate(data_file: Text, config_file: Text, folds: int) -> Dict:
@@ -1235,10 +1193,10 @@ def create_app(
         importer = TrainingDataImporter.load_from_dict(
             config=None, config_path=config_file, training_data_paths=[data_file]
         )
-        config = await importer.get_config()
-        nlu_data = await importer.get_nlu_data()
+        config = importer.get_config()
+        nlu_data = importer.get_nlu_data()
 
-        evaluations = rasa.nlu.test.cross_validate(
+        evaluations = await rasa.nlu.test.cross_validate(
             data=nlu_data,
             n_folds=folds,
             nlu_config=config,
@@ -1283,7 +1241,7 @@ def create_app(
         request_params = request.json
         try:
             tracker = DialogueStateTracker.from_dict(
-                DEFAULT_SENDER_ID, request_params, app.ctx.agent.domain.slots
+                DEFAULT_SENDER_ID, request_params, app.agent.domain.slots
             )
         except Exception as e:
             logger.debug(traceback.format_exc())
@@ -1295,9 +1253,7 @@ def create_app(
             )
 
         try:
-            result = app.ctx.agent.create_processor().predict_next_with_tracker(
-                tracker, verbosity
-            )
+            result = app.agent.predict_next_with_tracker(tracker, verbosity)
 
             return response.json(result)
         except Exception as e:
@@ -1323,9 +1279,7 @@ def create_app(
         try:
             data = emulator.normalise_request_json(request.json)
             try:
-                parsed_data = await app.ctx.agent.parse_message_using_nlu_interpreter(
-                    data.get("text")
-                )
+                parsed_data = await app.agent.parse_message(data.get("text"))
             except Exception as e:
                 logger.debug(traceback.format_exc())
                 raise ErrorResponse(
@@ -1366,13 +1320,14 @@ def create_app(
                     {"parameter": "model_server", "in": "body"},
                 )
 
-        app.ctx.agent = await _load_agent(
-            model_path,
-            model_server,
-            remote_storage,
-            endpoints,
-            app.ctx.agent.lock_store,
+        new_agent = await _load_agent(
+            model_path=model_path,
+            model_server=model_server,
+            remote_storage=remote_storage,
+            endpoints=endpoints,
         )
+        new_agent.lock_store = app.agent.lock_store
+        app.agent = new_agent
 
         logger.debug(f"Successfully loaded model '{model_path}'.")
         return response.json(None, status=HTTPStatus.NO_CONTENT)
@@ -1380,9 +1335,9 @@ def create_app(
     @app.delete("/model")
     @requires_auth(app, auth_token)
     async def unload_model(request: Request) -> HTTPResponse:
-        model_file = app.ctx.agent.model_directory
+        model_file = app.agent.model_name
 
-        app.ctx.agent = Agent(lock_store=app.ctx.agent.lock_store)
+        app.agent = Agent(lock_store=app.agent.lock_store)
 
         logger.debug(f"Successfully unloaded model '{model_file}'.")
         return response.json(None, status=HTTPStatus.NO_CONTENT)
@@ -1394,10 +1349,10 @@ def create_app(
         """Get current domain in yaml or json format."""
         accepts = request.headers.get("Accept", default=JSON_CONTENT_TYPE)
         if accepts.endswith("json"):
-            domain = app.ctx.agent.domain.as_dict()
+            domain = app.agent.domain.as_dict()
             return response.json(domain)
         elif accepts.endswith("yml") or accepts.endswith("yaml"):
-            domain_yaml = app.ctx.agent.domain.as_yaml()
+            domain_yaml = app.agent.domain.as_yaml()
             return response.text(
                 domain_yaml, status=HTTPStatus.OK, content_type=YAML_CONTENT_TYPE
             )
@@ -1438,7 +1393,7 @@ def _get_output_channel(
         requested_output_channel = tracker.get_latest_input_channel()
 
     # Interactive training does not set `input_channels`, hence we have to be cautious
-    registered_input_channels = getattr(request.app.ctx, "input_channels", None) or []
+    registered_input_channels = getattr(request.app, "input_channels", None) or []
     matching_channels = [
         channel
         for channel in registered_input_channels
@@ -1456,113 +1411,16 @@ def _get_output_channel(
     )
 
 
-def _test_data_file_from_payload(
-    request: Request, temporary_directory: Path, suffix: Text = ".tmp"
-) -> Text:
-    if request.headers.get("Content-type") == YAML_CONTENT_TYPE:
-        return str(
-            _training_payload_from_yaml(
-                request,
-                temporary_directory,
-                # test stories have to prefixed with `test_`
-                file_name=f"{TEST_STORIES_FILE_PREFIX}data.yml",
-            )["training_files"]
-        )
-    else:
-        # MD test stories have to be in the `tests` directory
-        test_dir = temporary_directory / DEFAULT_CONVERSATION_TEST_PATH
-        test_dir.mkdir()
-        test_file = test_dir / f"tests{suffix}"
-        test_file.write_bytes(request.body)
-        return str(test_file)
-
-
-def _training_payload_from_json(request: Request, temp_dir: Path) -> Dict[Text, Any]:
-    logger.debug(
-        "Extracting JSON payload with Markdown training data from request body."
+def _test_data_file_from_payload(request: Request, temporary_directory: Path) -> Text:
+    return str(
+        _training_payload_from_yaml(
+            request,
+            temporary_directory,
+            # test stories have to prefixed with `test_`
+            file_name=f"{TEST_STORIES_FILE_PREFIX}data.yml",
+        )["training_files"]
     )
 
-    request_payload = request.json
-    _validate_json_training_payload(request_payload)
-
-    config_path = os.path.join(temp_dir, "config.yml")
-
-    rasa.shared.utils.io.write_text_file(request_payload["config"], config_path)
-
-    if "nlu" in request_payload:
-        nlu_path = os.path.join(temp_dir, "nlu.md")
-        rasa.shared.utils.io.write_text_file(request_payload["nlu"], nlu_path)
-
-    if "stories" in request_payload:
-        stories_path = os.path.join(temp_dir, "stories.md")
-        rasa.shared.utils.io.write_text_file(request_payload["stories"], stories_path)
-
-    if "responses" in request_payload:
-        responses_path = os.path.join(temp_dir, "responses.md")
-        rasa.shared.utils.io.write_text_file(
-            request_payload["responses"], responses_path
-        )
-
-    domain_path = DEFAULT_DOMAIN_PATH
-    if "domain" in request_payload:
-        domain_path = os.path.join(temp_dir, "domain.yml")
-        rasa.shared.utils.io.write_text_file(request_payload["domain"], domain_path)
-
-    model_output_directory = str(temp_dir)
-    if request_payload.get(
-        "save_to_default_model_directory",
-        rasa.utils.endpoints.bool_arg(request, "save_to_default_model_directory", True),
-    ):
-        model_output_directory = DEFAULT_MODELS_PATH
-
-    return dict(
-        domain=domain_path,
-        config=config_path,
-        training_files=str(temp_dir),
-        output=model_output_directory,
-        force_training=request_payload.get(
-            "force", rasa.utils.endpoints.bool_arg(request, "force_training", False)
-        ),
-        core_additional_arguments=_extract_core_additional_arguments(request),
-        nlu_additional_arguments=_extract_nlu_additional_arguments(request),
-    )
-
-
-def _validate_json_training_payload(rjs: Dict) -> None:
-    if "config" not in rjs:
-        raise ErrorResponse(
-            HTTPStatus.BAD_REQUEST,
-            "BadRequest",
-            "The training request is missing the required key `config`.",
-            {"parameter": "config", "in": "body"},
-        )
-
-    if "nlu" not in rjs and "stories" not in rjs:
-        raise ErrorResponse(
-            HTTPStatus.BAD_REQUEST,
-            "BadRequest",
-            "To train a Rasa model you need to specify at least one type of "
-            "training data. Add `nlu` and/or `stories` to the request.",
-            {"parameters": ["nlu", "stories"], "in": "body"},
-        )
-
-    if "stories" in rjs and "domain" not in rjs:
-        raise ErrorResponse(
-            HTTPStatus.BAD_REQUEST,
-            "BadRequest",
-            "To train a Rasa model with story training data, you also need to "
-            "specify the `domain`.",
-            {"parameter": "domain", "in": "body"},
-        )
-
-    if "force" in rjs or "save_to_default_model_directory" in rjs:
-        rasa.shared.utils.io.raise_deprecation_warning(
-            "Specifying 'force' and 'save_to_default_model_directory' as part of the "
-            "JSON payload is deprecated. Please use the header arguments "
-            "'force_training' and 'save_to_default_model_directory'.",
-            docs=_docs("/api/http-api"),
-        )
-
 
 def _training_payload_from_yaml(
     request: Request, temp_dir: Path, file_name: Text = "data.yml"
