@@ -15,7 +15,6 @@ from typing import (
     ValuesView,
     Union,
     Sequence,
-    cast,
 )
 
 import rasa.shared.utils.io
@@ -31,6 +30,7 @@ from rasa.shared.core.events import (
     ActionExecuted,
     Event,
     SessionStarted,
+    SlotSet,
 )
 from rasa.shared.core.trackers import DialogueStateTracker
 from rasa.shared.exceptions import RasaCoreException
@@ -169,22 +169,32 @@ class StoryStep:
         return f"    - {story_step_element.as_story_string()}\n"
 
     @staticmethod
-    def _or_string(story_step_element: Sequence[Event], e2e: bool) -> Text:
+    def _event_to_story_string(event: Event, e2e: bool) -> Optional[Text]:
+        if isinstance(event, UserUttered):
+            return event.as_story_string(e2e=e2e)
+        return event.as_story_string()
+
+    @staticmethod
+    def _or_string(story_step_element: Sequence[Event], e2e: bool) -> Optional[Text]:
         for event in story_step_element:
-            if not isinstance(event, UserUttered):
+            # OR statement can also contain `slot_was_set`, and
+            # we're going to ignore this events when representing
+            # the story as a string
+            if not isinstance(event, UserUttered) and not isinstance(event, SlotSet):
                 raise EventTypeError(
-                    "OR statement events must be of type `UserUttered`."
+                    "OR statement events must be of type `UserUttered` or `SlotSet`."
                 )
 
-        # FIXME: https://github.com/python/mypy/issues/7853
-        story_step_element = cast(Sequence[UserUttered], story_step_element)
-
         result = " OR ".join(
-            [element.as_story_string(e2e) for element in story_step_element]
+            [
+                StoryStep._event_to_story_string(element, e2e)
+                for element in story_step_element
+            ]
         )
         return f"* {result}\n"
 
     def as_story_string(self, flat: bool = False, e2e: bool = False) -> Text:
+        """Returns a story as a string."""
         # if the result should be flattened, we
         # will exclude the caption and any checkpoints.
         if flat:
@@ -214,7 +224,9 @@ class StoryStep:
                 # The story reader classes support reading stories in
                 # conversion mode.  When this mode is enabled, OR statements
                 # are represented as lists of events.
-                result += self._or_string(event, e2e)
+                or_string = self._or_string(event, e2e)
+                if or_string:
+                    result += or_string
             else:
                 raise Exception(f"Unexpected element in story step: {event}")
 
@@ -254,7 +266,7 @@ class StoryStep:
 
     def explicit_events(
         self, domain: Domain, should_append_final_listen: bool = True
-    ) -> List[Union[Event, List[Event]]]:
+    ) -> List[Event]:
         """Returns events contained in the story step including implicit events.
 
         Not all events are always listed in the story dsl. This
@@ -262,8 +274,7 @@ class StoryStep:
         set slots. This functions makes these events explicit and
         returns them with the rest of the steps events.
         """
-
-        events = []
+        events: List[Event] = []
 
         for e in self.events:
             if isinstance(e, UserUttered):
@@ -473,13 +484,11 @@ class StoryGraph:
     def overlapping_checkpoint_names(
         cps: List[Checkpoint], other_cps: List[Checkpoint]
     ) -> Set[Text]:
-        """Find overlapping checkpoints names"""
-
+        """Find overlapping checkpoints names."""
         return {cp.name for cp in cps} & {cp.name for cp in other_cps}
 
     def with_cycles_removed(self) -> "StoryGraph":
         """Create a graph with the cyclic edges removed from this graph."""
-
         story_end_checkpoints = self.story_end_checkpoints.copy()
         cyclic_edge_ids = self.cyclic_edge_ids
         # we need to remove the start steps and replace them with steps ending
@@ -638,9 +647,8 @@ class StoryGraph:
     @staticmethod
     def _find_unused_checkpoints(
         story_steps: ValuesView[StoryStep], story_end_checkpoints: Dict[Text, Text]
-    ) -> Set[Text]:
+    ) -> Set[Optional[Text]]:
         """Finds all unused checkpoints."""
-
         collected_start = {STORY_END, STORY_START}
         collected_end = {STORY_END, STORY_START}
 
@@ -658,14 +666,6 @@ class StoryGraph:
 
         return self.step_lookup.get(step_id)
 
-    def as_story_string(self) -> Text:
-        """Convert the graph into the story file format."""
-
-        story_content = ""
-        for step in self.story_steps:
-            story_content += step.as_story_string(flat=False)
-        return story_content
-
     @staticmethod
     def order_steps(
         story_steps: List[StoryStep],
@@ -806,6 +806,10 @@ class StoryGraph:
 
         return not self.story_steps
 
+    def __repr__(self) -> Text:
+        """Returns text representation of object."""
+        return f"{self.__class__.__name__}: {len(self.story_steps)} story steps"
+
 
 def generate_id(prefix: Text = "", max_chars: Optional[int] = None) -> Text:
     """Generate a random UUID.
