@@ -1,8 +1,10 @@
 import asyncio
+import copy
 import logging
 import os
 import shutil
 import warnings
+from pathlib import Path
 from types import TracebackType
 from typing import (
     Any,
@@ -14,22 +16,41 @@ from typing import (
     Type,
     TypeVar,
     Union,
+    ContextManager,
+    Set,
 )
 
+from socket import SOCK_DGRAM, SOCK_STREAM
+import numpy as np
 import rasa.utils.io
 from rasa.constants import DEFAULT_LOG_LEVEL_LIBRARIES, ENV_LOG_LEVEL_LIBRARIES
-from rasa.shared.constants import DEFAULT_LOG_LEVEL, ENV_LOG_LEVEL
+from rasa.shared.constants import DEFAULT_LOG_LEVEL, ENV_LOG_LEVEL, TCP_PROTOCOL
 import rasa.shared.utils.io
 
 logger = logging.getLogger(__name__)
 
 T = TypeVar("T")
 
+EXPECTED_WARNINGS = [
+    # TODO (issue #9932)
+    (
+        np.VisibleDeprecationWarning,
+        "Creating an ndarray from ragged nested sequences.*",
+    ),
+    # cf. https://github.com/tensorflow/tensorflow/issues/38168
+    (
+        UserWarning,
+        "Converting sparse IndexedSlices.* to a dense Tensor of unknown "
+        "shape. This may consume a large amount of memory.",
+    ),
+    (UserWarning, "Slot auto-fill has been removed in 3.0 .*",),
+]
 
-class TempDirectoryPath(str):
-    """Represents a path to an temporary directory. When used as a context
-    manager, it erases the contents of the directory on exit.
 
+class TempDirectoryPath(str, ContextManager):
+    """Represents a path to an temporary directory.
+
+    When used as a context manager, it erases the contents of the directory on exit.
     """
 
     def __enter__(self) -> "TempDirectoryPath":
@@ -40,7 +61,7 @@ class TempDirectoryPath(str):
         _exc: Optional[Type[BaseException]],
         _value: Optional[Exception],
         _tb: Optional[TracebackType],
-    ) -> bool:
+    ) -> None:
         if os.path.exists(self):
             shutil.rmtree(self)
 
@@ -61,11 +82,22 @@ def read_global_config(path: Text) -> Dict[Text, Any]:
         return {}
 
 
-def set_log_level(log_level: Optional[int] = None) -> None:
-    """Set log level of Rasa and Tensorflow either to the provided log level or
-    to the log level specified in the environment variable 'LOG_LEVEL'. If none is set
-    a default log level will be used."""
+def configure_logging_and_warnings(
+    log_level: Optional[int] = None,
+    warn_only_once: bool = True,
+    filter_repeated_logs: bool = True,
+) -> None:
+    """Sets log levels of various loggers and sets up filters for warnings and logs.
 
+    Args:
+        log_level: The lo level to be used for the 'Rasa' logger. Pass `None` to use
+            either the environment variable 'LOG_LEVEL' if it is specified, or the
+            default log level otherwise.
+        warn_only_once: determines whether user warnings should be filtered by the
+            `warnings` module to appear only "once"
+        filter_repeated_logs: determines whether `RepeatedLogFilter`s are added to
+            the handlers of the root logger
+    """
     if not log_level:
         log_level = os.environ.get(ENV_LOG_LEVEL, DEFAULT_LOG_LEVEL)
         log_level = logging.getLevelName(log_level)
@@ -80,8 +112,33 @@ def set_log_level(log_level: Optional[int] = None) -> None:
 
     os.environ[ENV_LOG_LEVEL] = logging.getLevelName(log_level)
 
+    if filter_repeated_logs:
+        for handler in logging.getLogger().handlers:
+            handler.addFilter(RepeatedLogFilter())
+
+    _filter_warnings(log_level=log_level, warn_only_once=warn_only_once)
+
+
+def _filter_warnings(log_level: Optional[int], warn_only_once: bool = True) -> None:
+    """Sets up filters for warnings.
+
+    Args:
+        log_level: the current log level. Certain warnings will only be filtered out
+            if we're not in debug mode.
+        warn_only_once: determines whether user warnings should be filtered by the
+            `warnings` module to appear only "once"
+    """
+    if warn_only_once:
+        warnings.filterwarnings("once", category=UserWarning)
+    if log_level and log_level > logging.DEBUG:
+        for warning_type, warning_message in EXPECTED_WARNINGS:
+            warnings.filterwarnings(
+                "ignore", message=f".*{warning_message}", category=warning_type
+            )
+
 
 def update_apscheduler_log_level() -> None:
+    """Configures the log level of `apscheduler.*` loggers."""
     log_level = os.environ.get(ENV_LOG_LEVEL_LIBRARIES, DEFAULT_LOG_LEVEL_LIBRARIES)
 
     apscheduler_loggers = [
@@ -122,9 +179,14 @@ def update_tensorflow_log_level() -> None:
     logging.getLogger("tensorflow").propagate = False
 
 
-def update_sanic_log_level(log_file: Optional[Text] = None) -> None:
-    """Set the log level of sanic loggers to the log level specified in the environment
-    variable 'LOG_LEVEL_LIBRARIES'."""
+def update_sanic_log_level(
+    log_file: Optional[Text] = None,
+    use_syslog: Optional[bool] = False,
+    syslog_address: Optional[Text] = None,
+    syslog_port: Optional[int] = None,
+    syslog_protocol: Optional[Text] = None,
+) -> None:
+    """Set the log level to 'LOG_LEVEL_LIBRARIES' environment variable ."""
     from sanic.log import logger, error_logger, access_logger
 
     log_level = os.environ.get(ENV_LOG_LEVEL_LIBRARIES, DEFAULT_LOG_LEVEL_LIBRARIES)
@@ -145,18 +207,34 @@ def update_sanic_log_level(log_file: Optional[Text] = None) -> None:
         logger.addHandler(file_handler)
         error_logger.addHandler(file_handler)
         access_logger.addHandler(file_handler)
+    if use_syslog:
+        formatter = logging.Formatter(
+            "%(asctime)s [%(levelname)-5.5s] [%(process)d]" " %(message)s"
+        )
+        socktype = SOCK_STREAM if syslog_protocol == TCP_PROTOCOL else SOCK_DGRAM
+        syslog_handler = logging.handlers.SysLogHandler(
+            address=(syslog_address, syslog_port), socktype=socktype,
+        )
+        syslog_handler.setFormatter(formatter)
+        logger.addHandler(syslog_handler)
+        error_logger.addHandler(syslog_handler)
+        access_logger.addHandler(syslog_handler)
 
 
 def update_asyncio_log_level() -> None:
-    """Set the log level of asyncio to the log level specified in the environment
-    variable 'LOG_LEVEL_LIBRARIES'."""
+    """Set the log level of asyncio to the log level.
+
+    Uses the log level specified in the environment variable 'LOG_LEVEL_LIBRARIES'.
+    """
     log_level = os.environ.get(ENV_LOG_LEVEL_LIBRARIES, DEFAULT_LOG_LEVEL_LIBRARIES)
     logging.getLogger("asyncio").setLevel(log_level)
 
 
 def update_matplotlib_log_level() -> None:
-    """Set the log level of matplotlib to the log level specified in the environment
-    variable 'LOG_LEVEL_LIBRARIES'."""
+    """Set the log level of matplotlib to the log level.
+
+    Uses the log level specified in the environment variable 'LOG_LEVEL_LIBRARIES'.
+    """
     log_level = os.environ.get(ENV_LOG_LEVEL_LIBRARIES, DEFAULT_LOG_LEVEL_LIBRARIES)
     logging.getLogger("matplotlib.backends.backend_pdf").setLevel(log_level)
     logging.getLogger("matplotlib.colorbar").setLevel(log_level)
@@ -164,18 +242,6 @@ def update_matplotlib_log_level() -> None:
     logging.getLogger("matplotlib.ticker").setLevel(log_level)
 
 
-def set_log_and_warnings_filters() -> None:
-    """
-    Set log filters on the root logger, and duplicate filters for warnings.
-
-    Filters only propagate on handlers, not loggers.
-    """
-    for handler in logging.getLogger().handlers:
-        handler.addFilter(RepeatedLogFilter())
-
-    warnings.filterwarnings("once", category=UserWarning)
-
-
 def sort_list_of_dicts_by_first_key(dicts: List[Dict]) -> List[Dict]:
     """Sorts a list of dictionaries by their first key."""
     return sorted(dicts, key=lambda d: list(d.keys())[0])
@@ -245,6 +311,34 @@ def update_existing_keys(
     return updated
 
 
+def override_defaults(
+    defaults: Optional[Dict[Text, Any]], custom: Optional[Dict[Text, Any]]
+) -> Dict[Text, Any]:
+    """Override default config with the given config.
+
+    We cannot use `dict.update` method because configs contain nested dicts.
+
+    Args:
+        defaults: default config
+        custom: user config containing new parameters
+
+    Returns:
+        updated config
+    """
+    config = copy.deepcopy(defaults) if defaults else {}
+
+    if not custom:
+        return config
+
+    for key in custom.keys():
+        if isinstance(config.get(key), dict):
+            config[key].update(custom[key])
+            continue
+        config[key] = custom[key]
+
+    return config
+
+
 class RepeatedLogFilter(logging.Filter):
     """Filter repeated log records."""
 
@@ -265,47 +359,6 @@ class RepeatedLogFilter(logging.Filter):
         return False
 
 
-def run_in_loop(
-    f: Coroutine[Any, Any, T], loop: Optional[asyncio.AbstractEventLoop] = None
-) -> T:
-    """Execute the awaitable in the passed loop.
-
-    If no loop is passed, the currently existing one is used or a new one is created
-    if no loop has been started in the current context.
-
-    After the awaitable is finished, all remaining tasks on the loop will be
-    awaited as well (background tasks).
-
-    WARNING: don't use this if there are never ending background tasks scheduled.
-        in this case, this function will never return.
-
-    Args:
-       f: function to execute
-       loop: loop to use for the execution
-
-    Returns:
-        return value from the function
-    """
-
-    if loop is None:
-        try:
-            loop = asyncio.get_event_loop()
-        except RuntimeError:
-            loop = asyncio.new_event_loop()
-            asyncio.set_event_loop(loop)
-    result = loop.run_until_complete(f)
-
-    # Let's also finish all running tasks:
-    # fallback for python 3.6, which doesn't have asyncio.all_tasks
-    try:
-        pending = asyncio.all_tasks(loop)
-    except AttributeError:
-        pending = asyncio.Task.all_tasks()
-    loop.run_until_complete(asyncio.gather(*pending))
-
-    return result
-
-
 async def call_potential_coroutine(
     coroutine_or_return_value: Union[Any, Coroutine]
 ) -> Any:
@@ -322,3 +375,84 @@ async def call_potential_coroutine(
         return await coroutine_or_return_value
 
     return coroutine_or_return_value
+
+
+def directory_size_in_mb(
+    path: Path, filenames_to_exclude: Optional[List[Text]] = None
+) -> float:
+    """Calculates the size of a directory.
+
+    Args:
+        path: The path to the directory.
+        filenames_to_exclude: Allows excluding certain files from the calculation.
+
+    Returns:
+        Directory size in MiB.
+    """
+    filenames_to_exclude = filenames_to_exclude or []
+    size = 0.0
+    for root, _dirs, files in os.walk(path):
+        for filename in files:
+            if filename in filenames_to_exclude:
+                continue
+            size += (Path(root) / filename).stat().st_size
+
+    # bytes to MiB
+    return size / 1_048_576
+
+
+def copy_directory(source: Path, destination: Path) -> None:
+    """Copies the content of one directory into another.
+
+    Unlike `shutil.copytree` this doesn't raise if `destination` already exists.
+
+    # TODO: Drop this in favor of `shutil.copytree(..., dirs_exist_ok=True)` when
+    # dropping Python 3.7.
+
+    Args:
+        source: The directory whose contents should be copied to `destination`.
+        destination: The directory which should contain the content `source` in the end.
+
+    Raises:
+        ValueError: If destination is not empty.
+    """
+    if not destination.exists():
+        destination.mkdir(parents=True)
+
+    if list(destination.glob("*")):
+        raise ValueError(
+            f"Destination path '{destination}' is not empty. Directories "
+            f"can only be copied to empty directories."
+        )
+
+    for item in source.glob("*"):
+        if item.is_dir():
+            shutil.copytree(item, destination / item.name)
+        else:
+            shutil.copy2(item, destination / item.name)
+
+
+def find_unavailable_packages(package_names: List[Text]) -> Set[Text]:
+    """Tries to import all package names and returns the packages where it failed.
+
+    Args:
+        package_names: The package names to import.
+
+    Returns:
+        Package names that could not be imported.
+    """
+    import importlib
+
+    failed_imports = set()
+    for package in package_names:
+        try:
+            importlib.import_module(package)
+        except ImportError:
+            failed_imports.add(package)
+
+    return failed_imports
+
+
+def module_path_from_class(clazz: Type) -> Text:
+    """Return the module path of an instance's class."""
+    return clazz.__module__ + "." + clazz.__name__
